/**
  Draw a raytraced disk using an OpenGL shader.
  
  Dr. Orion Sky Lawlor, olawlor@acm.org, 2012-01-24 (Public Domain)
*/
#include "physics/world.h" /* physics::library and physics::object */
#include "ogl/glsl.h"

/* Just include library bodies here, for easy linking */
#include "physics/world.cpp" 
#include "physics/config.cpp"
#include "ogl/glsl.cpp"


/* A raytraced object */
class rayObject : public physics::object {
public:
	rayObject(void) 
		:physics::object(0.01)  /* <- our timestep, in seconds */
	{ }
	
	void simulate(physics::library &lib) { }
	
	void draw(physics::library &lib) {
		/* Load up shader */
		GLhandleARB prog=MAKE_PROGRAM_OBJECT3(
		/* vertex program */
			void main(void) {
			   myColor = gl_Color;
			   /* Include everything in onscreen position */
			   gl_Position=gl_ModelViewProjectionMatrix*gl_Vertex;
			   /* Don't project, but do modelview transforms */
			   location=(vec3)(gl_ModelViewMatrix*gl_Vertex);
			}
		,
		/* shared vertex/fragment code */
			uniform vec3 camera; // world coordinates of camera
			varying vec4 myColor;
			varying vec3 location; // world coordinates of our pixel
		,
		/* fragment program */
			void main(void) {
				vec3 C=camera; // origin of ray
				vec3 D=location-camera; // direction of ray
				
				vec3 N=normalize(vec3(1.0,0.0,1.0)); // normal of plane
				float a=dot(C,N); // from ray-plane intersection equation
				float b=dot(D,N);
				float t=-a/b;
				
				if (t<0) discard; // intersection is behind your head!
				vec3 P=C+t*D; // ray-object intersection point
				if (length(P)>1.2) discard; // trim plane to a disk
				
				// Lighting calculation
				vec3 L=normalize(vec3(0.3,-0.5,1.0));
				vec3 H=0.5*(L+normalize(-D)); // blinn "halfway" vector
				float specular=pow(clamp(dot(H,N),0.0,1.0),50.0); // phong highlight
				
				// Classic lambertian + specular lighting
				vec3 color=clamp(dot(N,L),0.0,1.0)*vec3(0.8,0.0,0.2)+specular*vec3(0.6);
				
				gl_FragColor.rgb=color;
				gl_FragColor.a=1.0; // opaque
			}
		);
		
		glUseProgramObjectARB(prog);
		unsigned int ul=glGetUniformLocationARB(prog,"camera");
		glUniform3fvARB(ul,1,camera);

		/* Draw proxy geometry--big enough to hit all the pixels we need */
		glColor4f(0.0f,1.0f,0.0f,0.8f);
		
		glutSolidSphere(1.7,4,8);
		/*
		glBegin(GL_QUADS);
		glVertex3f(-1e3,0,-1e3);
		glVertex3f(-1e3,0,+1e3);
		glVertex3f(+1e3,0,+1e3);
		glVertex3f(+1e3,0,-1e3);
		glEnd();
		*/
		
		glUseProgramObjectARB(0);
	}
};


/* Called create a new simulation */
void physics_setup(physics::library &lib) {
	lib.world->add(new rayObject());
	lib.background[0]=lib.background[1]=0.6;
	lib.background[2]=1.0; // light blue
}

/* Called every frame to draw everything */
void physics_draw_frame(physics::library &lib) {
	/* Point and line smoothing needs usual alpha blending */
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	/* Draw opaque white ground plane */
	float groundz=-1.0;
	float groundsize=30;
	glEnable(GL_DEPTH_TEST);
	glColor3f(1,1,1);
	glBegin(GL_QUADS);
	glVertex3f(-groundsize,-groundsize,groundz);
	glVertex3f(+groundsize,-groundsize,groundz);
	glVertex3f(+groundsize,+groundsize,groundz);
	glVertex3f(-groundsize,+groundsize,groundz);
	glEnd();
	
	/* No depth while drawing ground grid */
	glDisable(GL_DEPTH_TEST);
	glColor4f(0.0,0.0,0.0,1.0);
	glEnable(GL_LINE_SMOOTH);
	groundz-=0.001;
	glLineWidth(2.0);
	if (camera.z>=groundz) { /* no grid if you're underneath ground */
		glBegin(GL_LINES);
		for (float grid=-groundsize;grid<=+groundsize;grid++) {
			glVertex3f(-groundsize,grid,groundz); 
			glVertex3f(+groundsize,grid,groundz); 
			glVertex3f(grid,-groundsize,groundz); 
			glVertex3f(grid,+groundsize,groundz); 
		}
		glEnd();
	}
	
	/* Do depth while drawing objects */
	glEnable(GL_DEPTH_TEST);
	lib.world->draw(lib);
	
	
}


